﻿/*--------------------------------------------------------------------------------------+
|
|     $Source: MstnExamples/Elements/ManagedTextTableExample/MainForm.cs $
|
|  $Copyright: (c) 2015 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/

using System.Windows.Forms;
using System.Collections;
using System.Linq;

using Bentley.MstnPlatformNET;
using Bentley.DgnPlatformNET;
using Bentley.DgnPlatformNET.Elements;
using Bentley.MstnPlatform;
using Bentley.GeometryNET;
using Bentley.GeometryNET.Common;

namespace ManagedTextTableExample
{

/*=================================================================================**//**
* @bsiclass                                                               Bentley Systems
+===============+===============+===============+===============+===============+======*/
public partial class MainForm : Form
{
//Id of TextTable element that we will work with.
private ElementId       m_tableElementId;

//Text Style object, Id and height
private DgnTextStyle    m_testTextStyle;
private ElementId       m_textStyleId;
private double          m_textStyleHeight;
TableSeedHarvester      m_tableSeedHarvester;

    /*---------------------------------------------------------------------------------**//**
    * Constructor.
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    public MainForm(TableSeedHarvester tableSeedHarvester)
    {
        m_tableSeedHarvester = tableSeedHarvester;
        InitializeComponent();

        //Get or Create Text style.
        GetOrCreateTextStyle();

        //Create initial Text Table element.
        CreateTextTableElement();
    }

    /*---------------------------------------------------------------------------------**//**
    * Get Text Style if it exists or create otherwise.
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void GetOrCreateTextStyle()
    {
        DgnFile activeDgnFile = ManagedTextTableExample.GetActiveDgnFile();

        //If text style exists in file just get it. No need to create this each time.
        m_testTextStyle = DgnTextStyle.GetByName("TextTable Style", activeDgnFile);

        if (null == m_testTextStyle)
        {
            m_testTextStyle = new DgnTextStyle("TextTable Style", activeDgnFile);
            m_testTextStyle.SetProperty(TextStyleProperty.Width, 1000D);
            m_testTextStyle.SetProperty(TextStyleProperty.Height, 1000D);
            m_testTextStyle.Add(activeDgnFile);
        }

        m_textStyleId = m_testTextStyle.Id;
        m_testTextStyle.GetProperty(TextStyleProperty.Height, out m_textStyleHeight);
    }
    /// <summary>
    /// Make a TextTable from a supplied seed.
    /// </summary>
    /// <param name="sv">The SheetView that contains the table data.</param>
    /// <param name="tableSeedModel">DgnModel: may be null.</param>
    /// <param name="activeModel">The active DgnModel.</param>
    /// <param name="activeTextStyle">Active Text Style.</param>
    /// <param name="nRows">Table will have this number of rows.</param>
    /// <param name="nCols">Table will have this number of columns.</param>
    /// <returns>TextTable</returns>
    private TextTable TextTableFromSeed(DgnModel tableSeedModel, DgnModel activeModel, int nRows, int nCols, DgnTextStyle activeTextStyle)
{
  TextTable textTable = null;
  double textHeight = 1.0;
  activeTextStyle.GetProperty(TextStyleProperty.Height, out textHeight);
  if (null != tableSeedModel)
  {
    TextTable seed = TextTable.Create((uint)nRows, (uint)nCols, activeTextStyle.Id, textHeight,
                       tableSeedModel);
    System.Diagnostics.Debug.Assert(null != seed, "TextTable.Create from tableSeedModel failed");
    textTable = seed.CloneAsSeed((uint)nRows, (uint)nCols, activeModel);
    System.Diagnostics.Debug.Assert(null != textTable, "TextTable.CloneAsSeed failed");
    string s = $"Created table from seed model '{tableSeedModel.ModelName}'";
    MessageCenter.Instance.ShowMessage(MessageType.Info, s, s, MessageAlert.Balloon);
  }
  return textTable;
}
    /*---------------------------------------------------------------------------------**//**
    * Create TextTable element when this app is loaded.
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void CreateTextTableElement()
    {
        TextTable someTable = null;
        //  Next line assumes that you have a DGNLib that contains a model named 'Red Table'
        DgnModel tableSeed = GetTableSeed();
        if (null == tableSeed)
        {
            //  Create a table in the same way as the SDK ManagedTextTableExample
            someTable = TextTable.Create((uint)dgData.RowCount, (uint)dgData.ColumnCount, m_textStyleId, m_textStyleHeight, ManagedTextTableExample.GetActiveDgnModel());
        }
        else
        {
            //  Create a table from a seed found in a DGNLib
            someTable = TextTableFromSeed(tableSeed, ManagedTextTableExample.GetActiveDgnModel (), dgData.RowCount, dgData.ColumnCount, m_testTextStyle);
        }

        TextTableElement tableElement = new TextTableElement(someTable);
        tableElement.AddToModel();

        m_tableElementId = tableElement.ElementId;

        Bentley.MstnPlatformNET.Session.Instance.Keyin("FIT VIEW EXTENDED");
    }

    /*---------------------------------------------------------------------------------**//**
    * Update DataGridView rows/columns if rows/columns are added to or removed from TextTable
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void UpdateDataGridViewRowsAndColumns(TextTable tTable)
    {
    int rowDiff = dgData.RowCount - (int)tTable.RowCount;
    if (rowDiff > 0)
        {
        for (int i = 0; i < rowDiff; i++)
            dgData.Rows.RemoveAt(dgData.RowCount - 2);
        }
    else if (rowDiff < 0)
        {
        dgData.Rows.Add(System.Math.Abs(rowDiff));
        }

    int columnDiff = dgData.ColumnCount - (int)tTable.ColumnCount;
    if(columnDiff > 0)
        {
        for (int i = 0; i < columnDiff; i++)
            dgData.Columns.RemoveAt(dgData.ColumnCount - 1);
        }
    else if(columnDiff < 0)
        {
        for (int i = 0; i < System.Math.Abs(columnDiff); i++)
            {
            string columnName = "Column" + dgData.ColumnCount.ToString();
            dgData.Columns.Add(columnName, columnName);
            }
        }
    }

    /*---------------------------------------------------------------------------------**//**
    * Update DataGridView from MicroStation TextTable element.
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void TlbGetFrom_Click(object sender, System.EventArgs e)
    {
    Element elem = ManagedTextTableExample.GetActiveDgnModel().FindElementById(m_tableElementId);

    if(!elem.IsDeleted)
        {
        TextTableElement tableElem = (TextTableElement)elem;
        TextTable tTable = tableElem.ExtractTextTable();

        UpdateDataGridViewRowsAndColumns(tTable);

        for (uint row = 0; row < tTable.RowCount; row++)
            {
            for (uint column = 0; column < tTable.ColumnCount; column++)
                {
                TableCellIndex index = new TableCellIndex(row, column);
                TextTableCell cell = tTable.GetCell(index);

                TextBlock tb = cell.TextBlock;

                dgData.Rows[(int)row].Cells[(int)column].Value = (null == tb) ? "" : tb.ToString();
                }
            }
        }
    else
        {
        MessageBox.Show("Looks like the table has been deleted. Generating it again", "MicroStation SDK", MessageBoxButtons.OK, MessageBoxIcon.Error);

        CreateTextTableElement();
        }

    }

    /*---------------------------------------------------------------------------------**//**
    * Update a cell value in TextTable
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void PopulateCell(ref TextTable table, uint row, uint column, string value)
    {
        TableCellIndex index = new TableCellIndex(row, column);
        TextTableCell cell = table.GetCell(index);
        TextBlock tb = new TextBlock(m_testTextStyle, ManagedTextTableExample.GetActiveDgnModel());
        tb.AppendText(value);
        cell.TextBlock = tb;
    }

    /*---------------------------------------------------------------------------------**//**
    * Create new TextTable and replace the original TextTable element.
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void PopulateTextTable(ArrayList list)
    {
            
    TextTable table = TextTable.Create((uint)dgData.RowCount, (uint)dgData.ColumnCount, m_textStyleId, m_textStyleHeight, ManagedTextTableExample.GetActiveDgnModel());
    for (int i = 0; i < dgData.RowCount; i++)
        {
        ArrayList tData = (ArrayList)list[(int)i];
        for (int j = 0; j < dgData.ColumnCount; j++)
            {
            PopulateCell(ref table, (uint)i, (uint)j, tData[(int)j].ToString());
            }
        }
            
    TextTableElement newTableElem = new TextTableElement(table);

    Element elem = ManagedTextTableExample.GetActiveDgnModel().FindElementById(m_tableElementId);

    if (!elem.IsDeleted)
        {
        TextTableElement oldTableElem = (TextTableElement)elem;
        newTableElem.ReplaceInModel(oldTableElem);
        }
    else
        {
        MessageBox.Show("Looks like the table is removed. Generating it again.", "MicroStation SDK", MessageBoxButtons.OK, MessageBoxIcon.Error);
        newTableElem.AddToModel();

        m_tableElementId = newTableElem.ElementId;
        }
    }

    /*---------------------------------------------------------------------------------**//**
    * Update TextTable element from DataGridView
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void TlbSet_Click(object sender, System.EventArgs e)
    {
        ArrayList list = new ArrayList();
        foreach(DataGridViewRow row in dgData.Rows)
            {
            ArrayList rowData = new ArrayList();
            foreach(DataGridViewCell cell in row.Cells)
                {
                rowData.Add((null == cell.Value) ? "" : cell.Value.ToString());
                }
            list.Add(rowData);
            }

        PopulateTextTable(list);
        Bentley.MstnPlatformNET.Session.Instance.Keyin("FIT VIEW EXTENDED");
    }

    /*---------------------------------------------------------------------------------**//**
    * Add new column in DataGridView.
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void TlbAddColumn_Click(object sender, System.EventArgs e)
    {
        string columnName = "Column" + dgData.ColumnCount.ToString();
        dgData.Columns.Add(columnName, columnName);
    }

    /*---------------------------------------------------------------------------------**//**
    * Remove Column from DataGridView.
    * @bsimethod                                                              Bentley Systems
    /*--------------+---------------+---------------+---------------+---------------+------*/
    private void TlbRemoveColumn_Click(object sender, System.EventArgs e)
    {
    try
        {
        dgData.Columns.RemoveAt(dgData.CurrentCell.ColumnIndex);
        }
    catch
        {

        }
    }

        private void toolStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }
        /// <summary> Get a table seed from list of table seeds found in DGNLibs. </summary>
        /// <param name="name">Name of seed table to retrieve.</param>
        /// <returns>DgnModel</returns>
        DgnModel GetTableSeed(string name)
        {
            TableSeedInfo info = m_tableSeedHarvester.TableSeeds.Find(x => 0 == string.Compare(name, x.TableSeedName, true));
            return info.TableSeed;
        }
        /// <summary> Get a table seed from list of table seeds found in DGNLibs. </summary>
        /// <param name="index">Index into list of seeds.</param>
        /// <returns>DgnModel</returns>
        DgnModel GetTableSeed(int index = 0)
        {
            if (0 <= index && index < m_tableSeedHarvester.TableSeeds.Count)
            {
                TableSeedInfo info = m_tableSeedHarvester.TableSeeds [index];
                if (null == info || 0 == m_tableSeedHarvester.TableSeeds.Count)
                {
                    string s = "No table seed found in your DGNLibs";
                    MessageCenter.Instance.ShowMessage(MessageType.Warning, s, s, MessageAlert.Balloon);
                }
                else
                {
                    string s = $"Using table seed '{info.TableSeedName}' found in your DGNLibs";
                    MessageCenter.Instance.ShowMessage(MessageType.Info, s, s, MessageAlert.Balloon);
                    return info.TableSeed;
                }
            }
            else
            {
                throw new System.IndexOutOfRangeException($"Invalid index {index} to list of Table Seeds");
            }
            return null;
        }
    }
}
