﻿/*--------------------------------------------------------------------------------------+
|
|     $Source: MstnExamples/Elements/ManagedTextTableExample/ManagedTextTableExample.cs $
|
|  $Copyright: (c) 2020 LA Solutions Ltd. All rights reserved. $
|  $Copyright: (c) 2015 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/

using Bentley.DgnPlatformNET;
using Bentley.DgnPlatformNET.Elements;
using Bentley.MstnPlatformNET;
using System.Collections.Generic;
using System.Linq;

namespace ManagedTextTableExample
{
    /*=================================================================================**//**
    * @bsiclass                                                               Bentley Systems
    +===============+===============+===============+===============+===============+======*/
    [Bentley.MstnPlatformNET.AddInAttribute(MdlTaskID = "ManagedTextTableExample")]
    public sealed class ManagedTextTableExample : Bentley.MstnPlatformNET.AddIn
    {
        /// <summary>A static reference to this AddIn's instance. </summary>
        private static ManagedTextTableExample s_ManagedTextTableExample = null;
        /// <summary> Table Seed Harvester obtains a list of TextTable seed models from your attached DGNLibs. </summary>
        public TableSeedHarvester TableSeedHarvester { get; set; } = null;
        /// <summary> ManagedTextTableExample constructor. </summary>
        public ManagedTextTableExample(System.IntPtr mdlDesc)
        : base(mdlDesc)
        {
            TableSeedHarvester = new TableSeedHarvester();
            //  MainForm moved to Run()
            //MainForm mainForm = new MainForm(TableSeedHarvester);
            //mainForm.Show();
        }
        /// <summary> Helper method to get the active DGN file. </summary>
        public static DgnFile GetActiveDgnFile()
        {
            return Session.Instance.GetActiveDgnFile();
        }
        /// <summary> Helper method to get the active DGN model. </summary>
        public static DgnModel GetActiveDgnModel()
        {
            return Session.Instance.GetActiveDgnModel();
        }
        /// <summary> Required Run method of AddIn class.</summary>
        protected override int Run(string[] commandLine)
        {
            s_ManagedTextTableExample = this;
            TableSeedHarvester.GetTableSeedModels();
            TableSeedHarvester.ToString();
            //  MainForm moved from constructor
            MainForm mainForm = new MainForm(TableSeedHarvester);
            mainForm.Show();

            return 0;
        }
        /// <summary>Returns an instance of this Addin class.</summary>
        internal static ManagedTextTableExample Instance()
        {
            return s_ManagedTextTableExample;
        }
    }
    /// <summary>
    /// TableSeedInfo class stores information useful to finding and manipulating Table Seeds.
    /// </summary>
    public class TableSeedInfo
    {
        /// <summary>Name of file, typically a DGNLib, where Table Seed is located. </summary>
        public string DgnFileName { get; set; }
        /// <summary>Name of DGN model where Table Seed is located. </summary>
        public string TableSeedName { get { return TableSeed.ModelName; } }
        /// <summary>DGN model found in DGNLib. </summary>
        public DgnModel TableSeed { get; set; }
        /// <summary>Construct a TableSeedInfo instance. </summary>
        public TableSeedInfo(string file, DgnModel model) { DgnFileName = file; TableSeed = model; }
        /// <summary> Return a description of this Table Seed.</summary>
        /// <returns>Seed description.</returns>
        public new string ToString ()
        {
            return $"DGN file '{DgnFileName}' table seed '{TableSeedName}'";
        }
    }
    /// <summary>
    /// TableSeedHarvester finds Table Seed models in a DGN file, typically in a DGNLib.
    /// <remarks>A table seed is a DGN model having a single TextTable element.</remarks>
    /// </summary>
    public class TableSeedHarvester
    {
        private List<TableSeedInfo> seedInfo_;
        /// <summary>
        /// Return our list of Model Names.
        /// </summary>
        public List<TableSeedInfo> TableSeeds { get { return seedInfo_; } }
        /// <summary>
        /// Construct a TableSeedHarvester.
        /// </summary>
        public TableSeedHarvester()
        {
            seedInfo_ = new List<TableSeedInfo>();
        }
        /// <summary>
        /// Get a list of DGN models from visible DGNLibs.
        /// </summary>
        /// <returns>Count of models found</returns>
        public int GetTableSeedModels()
        {
            DgnLibIterator iterator = new DgnLibIterator(DgnLibSelector.ElementStyles);
            foreach (DgnFile lib in iterator)
            {
                var indices = lib.GetModelIndexCollection();
                foreach (var index in indices)
                {
                    StatusInt status;
                    DgnModel model = lib.LoadRootModelById(out status, index.Id);
                    if (IsTableSeed(model))
                    {
                        string s = $"Table seed: {lib.GetFileName()}.{model.ModelName}";
                        MessageCenter.Instance.ShowMessage(MessageType.Debug, s, s, MessageAlert.None);
                        seedInfo_.Add(new TableSeedInfo(lib.GetFileName(), model));
                    }
                }
            }
            return seedInfo_.Count;
        }
        /// <summary> Determine if a DGN model contains a Table Seed.</summary>
        /// <param name="model">DGN model to test.</param>
        /// <returns>True if DGN model contains a single element that is a Table.</returns>
        private bool IsTableSeed(DgnModel model)
        {
            var elements = model.GetGraphicElements();
            return (elements.Count() == 1 && elements.First() is TextTableElement);
        }
        /// <summary>
        /// Get the Table Seed chosen by the user.
        /// </summary>
        /// <param name="model">DGN model containing Table Seed.</param>
        /// <param name="info">TableSeedInfo containing DGN file and model name.</param>
        /// <returns></returns>
        public bool GetTableSeed(ref DgnModel model, TableSeedInfo info)
        {
            bool found = true;
            //  Default value is active model.
            if (null == info)
            {
                model = Session.Instance.GetActiveDgnModel();
            }
            else
            {

            }
            return found;
        }
        public void Trace ()
        {
            string s = $"There are {TableSeeds.Count} table seeds";
            MessageCenter.Instance.ShowMessage(MessageType.Debug, s, s, MessageAlert.None);
            foreach (var info in TableSeeds)
            {
                MessageCenter.Instance.ShowMessage(MessageType.Debug, info.ToString (), info.ToString(), MessageAlert.None);
            }
        }
    }
}
